## dependenceees

A React Higher Order Component which injects dependencies into the DOM.

### Quick start

Install

```sh
$ yarn add @lodgify/dependenceees
```

Use

```js
import React from 'react';
import {Dependenceees} from '@lodgify/dependenceees';

const scripts = [{
  src: 'https://cdnjs.com/someAwesomeLibrary.min.js',
  id: 'some-awesome-library'
}];

const TodoList = () => (
  <Dependenceees
    scripts={scripts}
  >
    <WaitingForScripts />
  </Dependenceees>
)

```

### API

##### @prop {Object[]} `links`

Each object in the `links` array should describe the attributes of a `<link ... />` element for `Dependenceees` to append to `document.head`.

You can specify a function as the `check` attribute. If `check` returns a truthy value, `Dependenceees` will skip the link. The `check` attribute is not added to the element injected into the DOM.

**Example**

```js
const links = [
  {
    href: '//styles.com/wow.css',
    type: 'text/css',
    rel: 'stylesheet',
    // optional: skip this link if it is already in the DOM
    check: () => document.querySelector('link[href="//styles.com/wow.css"]')
  }
];

const Component = () => <Dependenceees links={links}/>;

```

produces

```html
<head>
  ...
  <link rel="stylesheet" type="text/css" href="//styles.com/wow.css">
</head>
```

##### @prop {Object[]} `scripts`

Each object in the `scripts` array should describe the attributes of a `<script ... ></script>` element for `Dependenceees` to append to `document.body`.

You can specify a function as the `check` attribute. If `check` returns a truthy value, `Dependenceees` will skip the script. The `check` attribute is not added to the element injected into the DOM.

**Example**

```js
const scripts = [
  {
    src: '//scripts.com/doStuff.js',
    id: 'do-stuff',
    // optional: skip this script if the feature is detected
    check: () => global.doStuff
  }
];

const Component = () => <Dependenceees scripts={scripts}/>;

```

produces

```html
<body>
  ...
  <script id="do-stuff" src="//scripts.com/doStuff.js"></script>
</body>
```

##### @prop {Boolean} `shouldLoadAsync`

If `true`, `Dependenceees` leaves the browser to load and evaluate scripts asynchronously. Use this option if you don't care about the order in which your scripts are evaluated and you want to speed up loading.

`Dependenceees` defaults to injecting scripts synchronously preserving the order of the `props.scripts` array.

##### @prop {Object[]} `domData`

Each object in the `domData` array should describe the attributes of a `<div ... ></div>` element for `Dependenceees` to append to `document.body`.

You can specify a function as the `check` attribute. If `check` returns a truthy value, `Dependenceees` will skip the element. The `check` attribute is not added to the element injected into the DOM.

**Example**

```js
const domData = [
  {
    id: 'some-data',
    hidden: 'true',
    dataset: {
      some: 'data'
    }
    // optional: skip this element if it is already in the DOM
    check: () => document.querySelector('div#some-data')
  }
];

const Component = () => <Dependenceees domData={domData}/>;

```

produces

```html
<body>
  ...
  <div id="some-data" hidden data-some="data"></div>
</body>
```

##### @prop {Function} `onLoaded`

A function to execute once `Dependenceees` has finished the synchronous load of the last dependency in `props.scripts`.

If either:

1. only `props.links` and/or `props.domData` are passed, or
1. `props.scripts` are passed and `props.async` is `true`,

`onLoaded` is called immediately after injection of elements into the DOM and is not associated with any load event.


##### @prop {ReactElement|ReactElement[]} `children`

Children of `Dependenceees` will be rendered after `onLoaded` is called.


##### @prop {Function} `onChildrenRendered`

A function to execute once the children of `Dependenceees` have been rendered.
