import {
  createElementWithAttributes,
  injectElements,
} from './injectElements';

describe('utils/injectElements', () => {
  describe('createElementWithAttributes', () => {
    test('should return an element of the required tagName', () => {
      expect(
        createElementWithAttributes({}, 'script')
      ).toEqual(expect.any(HTMLScriptElement));
    });

    test('should return a `div` by default', () => {
      expect(createElementWithAttributes({})).toEqual(
        expect.objectContaining({
          tagName: 'DIV',
        })
      );
    });

    test('should populate the element with attributes', () => {
      const mockAttributes = {
        id: 'someId',
        anotherAttr: 'yo',
      };

      expect(
        createElementWithAttributes(mockAttributes)
      ).toEqual(expect.objectContaining(mockAttributes));
    });

    test('should skip the `check` property', () => {
      const mockAttributes = { check: () => {} };

      expect(
        createElementWithAttributes(mockAttributes)
      ).not.toEqual(
        expect.objectContaining(mockAttributes)
      );
    });

    test('should iterate over the `dataset` property', () => {
      // Naive implementation of `HTMLElement.prototype.dataset`
      HTMLElement.prototype.dataset = {};

      const mockAttributes = {
        dataset: {
          cogombre: 'pepino',
          carxofa: 'alcachofa',
        },
      };

      expect(
        createElementWithAttributes(mockAttributes)
      ).toEqual(expect.objectContaining(mockAttributes));
    });
  });

  describe('injectElements', () => {
    test('should call `elements.forEach` with a function', () => {
      const forEach = jest.fn();
      const elements = { forEach };

      injectElements(elements, () => {});

      expect(forEach.mock.calls.length).toBe(1);

      expect(forEach.mock.calls[0][0]).toEqual(
        expect.any(Function)
      );
    });

    test('should default to append an element to the body', () => {
      injectElements([{ id: 'pickMe' }]);

      expect(
        document.querySelector('body #pickMe')
      ).toBeTruthy();
    });

    test('should skip elements with passing checks', () => {
      const elements = [{ id: 'nope', check: () => true }];

      injectElements(elements, () => {});

      expect(document.querySelector('#nope')).toBeFalsy();
    });

    test('should add elements with failing checks', () => {
      const elements = [{ id: 'yup', check: () => false }];

      injectElements(elements);

      expect(document.querySelector('#yup')).toBeTruthy();
    });

    test('should add `link` elements to the head', () => {
      injectElements([{ id: 'iAmLink' }], 'link');

      expect(
        document.querySelector('head #iAmLink')
      ).toBeTruthy();
    });

    test('should call `done` if present', () => {
      const scripts = { forEach: () => {} };
      const done = jest.fn();

      injectElements(scripts, undefined, done);

      expect(done.mock.calls.length).toBe(1);
    });
  });
});
