import {
  syncAppendScripts,
  injectScripts,
} from './injectScripts';

describe('utils/injectScripts.js', () => {
  beforeAll(() => {
    const appendChildCopy = document.body.appendChild.bind(
      document.body
    );

    document.body.appendChild = element => {
      appendChildCopy(element);
      // Force the `onload` behaviour
      const load = new Event('load');
      element.dispatchEvent(load);
    };
  });

  describe('syncAppendScripts', () => {
    test('should default to append a `script` element for each item in `scripts`', () => {
      const scripts = [
        { class: 'meMeMe' },
        { class: 'meMeMe' },
      ];

      syncAppendScripts(scripts, () => {});

      expect(
        Array.from(document.body.children).filter(
          element => element.class === 'meMeMe'
        ).length
      ).toBe(2);
    });

    test('should skip `script` elements with passing checks`', () => {
      const scripts = [
        { class: 'noooo', check: () => true },
        { class: 'noooo' },
      ];

      syncAppendScripts(scripts, () => {});

      expect(
        Array.from(document.body.children).filter(
          element => element.class === 'noooo'
        ).length
      ).toBe(1);
    });

    test('should include `script` elements with failing checks`', () => {
      const scripts = [
        { class: 'whoop', check: () => false },
        { class: 'whoop' },
      ];

      syncAppendScripts(scripts, () => {});

      expect(
        Array.from(document.body.children).filter(
          element => element.class === 'whoop'
        ).length
      ).toBe(2);
    });

    test('should set an `onload` function', () => {
      expect(
        Array.from(document.body.children).filter(
          element => element.class === 'meMeMe'
        )[0]
      ).toEqual(
        expect.objectContaining({
          onload: expect.any(Function),
        })
      );
    });

    test('should call `done`', () => {
      const scripts = [{}];
      const done = jest.fn();

      syncAppendScripts(scripts, done);

      expect(done.mock.calls.length).toBe(1);
    });
  });

  describe('injectScripts', () => {
    test('should call `asyncAppendScripts` if `async` is true', () => {
      const forEach = jest.fn();
      const scripts = { forEach };

      const done = jest.fn();

      injectScripts(scripts, true, done);

      // If `forEach` has been called
      // and `done` has been called,
      // `asyncAppendScripts` has been called.
      expect(forEach.mock.calls.length).toBe(1);
      expect(done.mock.calls.length).toBe(1);
    });

    test('should call `injectElements` if `async` is falsy', () => {
      const forEach = jest.fn();
      const scripts = [{}];
      scripts.forEach = forEach;

      const done = jest.fn();

      injectScripts(scripts, undefined, done);

      // If `forEach` has not been called
      // and `done` has been called,
      // `syncAppendScripts` has been called.
      expect(forEach.mock.calls.length).toBe(0);
      expect(done.mock.calls.length).toBe(1);
    });
  });
});
