import isNumber from 'lodash/isNumber';
import isString from 'lodash/isString';

/**
 * Returns an `string` with the correct keyName, pluralized if proceeds.
 * It uses early return at any different case
 * React components using it validates quantity as a Number, but checking is needed though
 * @param  {String} keyName
 * @param  {String} pluralKeyName
 * @param  {Any} quantity (although it should come always Number)
 * @return {String}
 */
export const getKeyGivenPlurarity = (
  keyName,
  pluralKeyName = '',
  quantity = 0
) => {
  // case where pluralKeyName hasn't been informed
  if (!pluralKeyName.length) {
    return keyName;
  }

  // regardless keyNames, if no quantity provided, singular keyName returned
  if (!isNumber(quantity)) {
    return keyName;
  }

  // case where pluralKeyName has been informed but singular still applies
  if (quantity === 1 || quantity === -1) {
    return keyName;
  }

  // plural case
  return pluralKeyName;
};

export const getTranslation = (
    translations = {},
    customTranslations = {},
    shortCode,
    keyName,
    pluralKeyName = '',
    quantity = 0
  ) => {
  const processedKeyName = getKeyGivenPlurarity(keyName, pluralKeyName, quantity);

  if (customTranslations[processedKeyName]) {
    return customTranslations[processedKeyName];
  }
  if (translations[shortCode] && translations[shortCode][processedKeyName]) {
    return translations[shortCode][processedKeyName];
  }
  return processedKeyName;
};

/**
 * Returns an `string` with the correct translation, if exists, pluralized if proceeds.
 *
 * @param  {Object} translations
 * @param  {Object} customTranslations
 * @param  {String} shortCode
 * @param  {String} keyName
 * @param  {String} pluralKeyName
 * @param  {Number} quantity
 * @param  {Array}  values
 * @return {String}
 */
export const translate = (
    translations = {},
    customTranslations = {},
    shortCode,
    keyName,
    pluralKeyName = '',
    quantity = 0,
    values = []
  ) => {
  if (isString(keyName)) {
    return getTranslation(translations, customTranslations, shortCode, keyName, pluralKeyName, quantity);
  }

  // Case whereas keyName is an array of strings
  let basedTranslation = getTranslation(translations, customTranslations, shortCode, keyName[0]);
  const addedTranslation = getTranslation(translations, customTranslations, shortCode, keyName[1]);
  basedTranslation = basedTranslation.replace('{0}', addedTranslation);

  for (const [index, value] of values.entries()) {
    basedTranslation = basedTranslation.replace(`{${(index + 1)}}`, value);
  }

  return basedTranslation;
};
