import React, { Component } from 'react';
import PropTypes from 'prop-types';
import styled from 'styled-components';
import deviceDetect from 'device-detect';

import ImageGallery from 'react-image-gallery';
import 'react-image-gallery/styles/css/image-gallery.css';

import styles from './SlideShow.scss';

const SlideShowContainer = styled.section`
  overflow: hidden;
  position: relative;
  min-height: 400px;
  height: 100%;

  & img {
    height: 100%;
    object-fit: cover;
    width: 100%;
    max-width: 100%;
  }
  .image-gallery-image {
    height: 100%;
  }
`;

const RightNavButton = styled.span`
  position: absolute;
  top: 50%;
  right: 30px;
  transform: translateY(-50%);
  background: ${props => props.theme.palette.action};
  width: 40px;
  height: 40px;
  border-radius: 100%;
  z-index: 1;
  cursor: pointer;
  &:after {
    content: '';
    position: absolute;
    top: 50%;
    left: 15px;
    transform-origin: right;
    transform: translateY(-50%) rotate(45deg);
    width: 10px;
    height: 1px;
    background-color: #010101;
  }
  &:before {
    content: '';
    position: absolute;
    top: 50%;
    left: 15px;
    transform-origin: right;
    transform: translateY(-50%) rotate(-45deg);
    width: 10px;
    height: 1px;
    background-color: #010101;
  }
`;
const LeftNavButton = RightNavButton.extend`
  left: 30px;
  &:before {
    right: 15px;
    transform-origin: left;
  }
  &:after {
    right: 15px;
    transform-origin: left;
  }
`;

class SlideShow extends Component {
  constructor(props) {
    super(props);
    this.handleImagesLoad = this.handleImagesLoad.bind(this);
    this.state = {
      images: [],
      loader: true,
      parallax: '',
    };
    this.renderLeftNav = this.renderLeftNav.bind(this);
    this.renderRightNav = this.renderRightNav.bind(this);
    this.useParallax = this.useParallax.bind(this);
  }

  componentDidMount() {
    this.handleImagesLoad();
    if (this.props.hasParallax) {
      this.useParallax();
    }
    const device = deviceDetect().device;
    this.isMobile = device !== 'Macintosh' && device !== 'Windows' && device !== 'Linux';
  }

  handleImagesLoad() {
    const images = [];
    this.props.images.map(image => {
      const imageObject = {
        original: image.url.replace(/lodgifydevelop/, 'lodgify'),
        thumbnail: '',
        originalTitle: '',
        srcSet: `${image.url.replace(/lodgifydevelop/, 'lodgify')}&w=400&h=400&mode=max 320w,
        ${image.url.replace(/lodgifydevelop/, 'lodgify')}&w=800&h=800&mode=max 768w,
        ${image.url.replace(/lodgifydevelop/, 'lodgify')}&w=1024&h=1000&mode=max 1024w`,
        sizes: '100w',
      };
      images.push(imageObject);
      return images;
    });
    if (images.length > 0) {
      this.setState({
        loader: false,
        images,
      });
    }
  }

  useParallax() {
    const speed = 0.5;
    const scale = 0.15;
    const xScrollPosition = global.scrollX;
    const windowYOffset = global.pageYOffset;
    const normalizedOffset = windowYOffset / global.innerHeight;
    const scaleValue = Math.max(1, 1 + normalizedOffset * scale);
    const translateY = windowYOffset * speed;
    this.setState({
      parallax: `translate3d(${xScrollPosition}px, ${translateY}px, 0) scale(${scaleValue}`,
    });
    global.requestAnimationFrame(this.useParallax);
  }

  renderLeftNav(onClick, disabled) {
    if (!this.isMobile) {
      return <LeftNavButton disabled={disabled} onClick={onClick} />;
    }
    return null;
  }

  renderRightNav(onClick, disabled) {
    if (!this.isMobile) {
      return (
        <RightNavButton
          className="image-gallery-custom-right-nav"
          disabled={disabled}
          onClick={onClick}
        />
      );
    }
    return null;
  }

  render() {
    return (
      <SlideShowContainer className="slideshow-widget">
        <div className={styles.parallaxContainer} style={{ transform: this.state.parallax }}>
          <ImageGallery
            lazyLoad
            showFullscreenButton={false}
            showBullets
            className={styles.sliderContainer}
            items={this.state.images}
            onImageLoad={this.handleImagesLoad}
            showThumbnails={false}
            autoPlay
            hasDots
            renderRightNav={this.renderRightNav}
            renderLeftNav={this.renderLeftNav}
            hasCTA
          />
        </div>
      </SlideShowContainer>
    );
  }
}

SlideShow.defaultProps = {
  hasParallax: false,
  images: [
    {
      text: null,
      guid: 'f42f6441-593b-41ed-9498-a7f4064d9d65',
      width: 0,
      height: 0,
      type: 'Jpeg',
      url: '//si5.cdbcdn.com/oh/f42f6441-593b-41ed-9498-a7f4064d9d65.jpg?f=32',
      context: null,
    },
    {
      text: null,
      guid: '0f934bff-735a-4283-95bf-a8c3a0ecf6ea',
      width: 0,
      height: 0,
      type: 'Jpeg',
      url: '//si4.cdbcdn.com/oh/0f934bff-735a-4283-95bf-a8c3a0ecf6ea.jpg?f=32',
      context: null,
    },
    {
      text: null,
      guid: '52edd7bf-a79d-4e71-9d0d-6793e03ca527',
      width: 0,
      height: 0,
      type: 'Jpeg',
      url: '//si3.cdbcdn.com/oh/52edd7bf-a79d-4e71-9d0d-6793e03ca527.jpg?f=32',
      context: null,
    },
    {
      text: null,
      guid: '59cc6f80-8e28-4793-a114-53f71523a216',
      width: 0,
      height: 0,
      type: 'Jpeg',
      url: '//si2.cdbcdn.com/oh/59cc6f80-8e28-4793-a114-53f71523a216.jpg?f=32',
      context: null,
    },
    {
      text: null,
      guid: 'b9730b2f-c185-4a68-a33a-a72fa265888c',
      width: 0,
      height: 0,
      type: 'Jpeg',
      url: '//si1.cdbcdn.com/oh/b9730b2f-c185-4a68-a33a-a72fa265888c.jpg?f=32',
      context: null,
    },
  ],
};

SlideShow.propTypes = {
  images: PropTypes.arrayOf(PropTypes.any).isRequired,
  hasParallax: PropTypes.bool.isRequired,
};

export default SlideShow;
