/* eslint-disable class-methods-use-this */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import styled from 'styled-components';
import MediaQuery from 'react-responsive';
import deviceDetect from 'device-detect';

import ImageGallery from 'react-image-gallery';
import 'react-image-gallery/styles/css/image-gallery.css';
import { CtaContainer, BookNowButton } from '../BookNowButton';
import HeroText from '../HeroText/HeroText';

import styles from './SlideShow.scss';

const SlideShowContainer = styled.section`
  height: 600px;
  overflow: hidden;
  margin-bottom: 40px;
  position: relative;
  div[class^='image'] {
    height: 100%;
  }
  @media screen and (max-width: 600px) {
    height: 400px;
  }
  & img {
    height: 100%;
    object-fit: cover;
    width: 100%;
    max-width: 100%;
    @media screen and (max-width: 600px) {
      height: 400px;
      object-fit: cover;
    }
  }
`;

const RightNavButton = styled.span`
  position: absolute;
  top: 50%;
  right: 30px;
  transform: translateY(-50%);
  background: ${props => props.theme.palette.action};
  width: 40px;
  height: 40px;
  border-radius: 100%;
  z-index: 1;
  cursor: pointer;
  &:after {
    content: '';
    position: absolute;
    top: 50%;
    left: 15px;
    transform-origin: right;
    transform: translateY(-50%) rotate(45deg);
    width: 10px;
    height: 1px;
    background-color: #010101;
  }
  &:before {
    content: '';
    position: absolute;
    top: 50%;
    left: 15px;
    transform-origin: right;
    transform: translateY(-50%) rotate(-45deg);
    width: 10px;
    height: 1px;
    background-color: #010101;
  }
`;

const LeftNavButton = RightNavButton.extend`
  left: 30px;
  &:before {
    right: 15px;
    transform-origin: left;
  }
  &:after {
    right: 15px;
    transform-origin: left;
  }
`;

class SlideShow extends Component {
  constructor(props) {
    super(props);
    this.handleImagesLoad = this.handleImagesLoad.bind(this);
    this.state = {
      images: [],
      loader: true,
      parallax: '',
    };
    this.renderLeftNav = this.renderLeftNav.bind(this);
    this.renderRightNav = this.renderRightNav.bind(this);
    this.useParallax = this.useParallax.bind(this);
  }

  componentDidMount() {
    this.handleImagesLoad();
    if (this.props.hasParallax) {
      this.useParallax();
    }
    const device = deviceDetect().device;
    this.isMobile = device !== 'Macintosh' && device !== 'Windows' && device !== 'Linux';
  }

  handleImagesLoad() {
    const images = [];
    this.props.images.map(image => {
      const imageObject = {
        original: image.url.replace(/lodgifydevelop/, 'lodgify'),
        thumbnail: '',
        originalTitle: '',
        srcSet: `${image.url.replace(/lodgifydevelop/, 'lodgify')}&w=400&h=400&mode=max 320w,
        ${image.url.replace(/lodgifydevelop/, 'lodgify')}&w=800&h=800&mode=max 768w,
        ${image.url.replace(/lodgifydevelop/, 'lodgify')}&w=1024&h=1000&mode=max 1024w`,
        sizes: '100w',
      };
      images.push(imageObject);
      return images;
    });
    if (images.length > 0) {
      this.setState({
        loader: false,
        images,
      });
    }
  }

  useParallax() {
    const speed = 0.5;
    const scale = 0.15;
    const xScrollPosition = global.scrollX;
    const windowYOffset = global.pageYOffset;
    const normalizedOffset = windowYOffset / global.innerHeight;
    const scaleValue = Math.max(1, 1 + normalizedOffset * scale);
    const translateY = windowYOffset * speed;
    this.setState({
      parallax: `translate3d(${xScrollPosition}px, ${translateY}px, 0) scale(${scaleValue}`,
    });
    global.requestAnimationFrame(this.useParallax);
  }

  renderLeftNav(onClick, disabled) {
    if (!this.isMobile) {
      return <LeftNavButton disabled={disabled} onClick={onClick} />;
    }
    return null;
  }

  renderRightNav(onClick, disabled) {
    if (!this.isMobile) {
      return (
        <RightNavButton
          className="image-gallery-custom-right-nav"
          disabled={disabled}
          onClick={onClick}
        />
      );
    }
    return null;
  }

  render() {
    if (this.props.hasThemeWidgets && this.props.images.length > 0) {
      return (
        <SlideShowContainer
          className="slideshow-home"
          style={this.props.styles ? this.props.styles : { height: '600px' }}
        >
          <div className={styles.parallaxContainer} style={{ transform: this.state.parallax }}>
            <ImageGallery
              lazyLoad
              className={styles.sliderContainer}
              items={this.state.images}
              onImageLoad={this.handleImagesLoad}
              showThumbnails={false}
              autoPlay
              renderRightNav={this.renderRightNav}
              renderLeftNav={this.renderLeftNav}
              showPlayButton={false}
              showFullscreenButton={false}
            />
          </div>
          <HeroText title={this.props.data.title} />
          <MediaQuery maxWidth={600} values={{ width: 600 }}>
            {this.props.hasCTA && (
              <CtaContainer>
                <BookNowButton>Book Now</BookNowButton>
              </CtaContainer>
            )}
          </MediaQuery>
        </SlideShowContainer>
      );
    }
    return null;
  }
}

SlideShow.defaultProps = {
  images: [],
  styles: {},
  hasCTA: false,
  data: {},
};

SlideShow.propTypes = {
  data: PropTypes.objectOf(PropTypes.any),
  images: PropTypes.arrayOf(PropTypes.any),
  styles: PropTypes.objectOf(PropTypes.any),
  hasThemeWidgets: PropTypes.bool.isRequired,
  hasParallax: PropTypes.bool.isRequired,
  hasCTA: PropTypes.bool,
};

export default SlideShow;
